/*
Represents a view with a time text and 6 arrows to adjust the shown time.

Props
	main: JQ -- wraps everything
	box: JQ -- forms white background.
	arrows: [JQ] -- Indices 0 - 2 correspond to top row, 3 - 5 are bottom.
	onKeyDown(DigitalView v, Event event) -- convenience handler.
	onInputMouseDown(DigitalView v, Event event) -- convenience handler.
	onInputFocusOut(DigitalView v, Event event) -- convenience handler.
	onArrowMouseDown(DigitalView v, Event event) -- convenience handler.
	onArrowMouseUp(DigitalView v, Event event) -- convenience handler.
	onArrowMouseLeave(DigitalView v, Event event) -- convenience handler.
	arrowMarg: Number -- vertical margin between arrows and box.
	_regTextColor : String -- text color when not in editing mode.

Mapped Index
	- This value represents the cursor position in terms of the input text without colons.
	- It can make a few of the calculations/manipulations easier
 */

var DigitalView = function() {
	//Props
	this.main = $('<div/>')
		.addClass('digital-view')
	this.box = $('<div/>')
		.addClass('digital-box')
		.appendTo(this.main);
	this.inputs = [];
	this.colon1 = this.createColon();
	this.colon2 = this.createColon();
	this.arrows = [];
	this.onArrowMouseDown = null;
	this.onArrowMouseUp = null;
	this.onArrowMouseLeave = null;
	this.onKeyDown = null;
	this.onInputMouseDown = null;
	this.onInputFocusOut = null;
	this.arrowMarg = 0;
	this._arrowsShown = true;
	this._regTextColor = 'white'; 

	//Init
	var i;
	for(i = 0; i < 6; i++) {
		this.arrows[i] = this.createArrow(i < 3);
	}
	for(i = 2; i >= 0; i--) { //append in backwards order
		this.inputs[i] = this.createInput();
	}
	this.setTime(0, 0, 0);
	this.setFontSize(10);
};

DigitalView.prototype = {

	//Init
	createArrow: function(pointUp) {
		var img1 = $('<img/>')
			.attr('src', pointUp ? 'img/up.png' : 'img/down.png')
			.attr('draggable', false)
			.width('70%')
			.height('70%')
			.addClass('centered');
		var img2 = $('<img/>')
			.attr('src', pointUp ? 'img/up2.png' : 'img/down2.png')
			.attr('draggable', false)
			.width('70%')
			.height('70%')
			.addClass('centered');
		var arrow = $('<div/>')
			.addClass('centered-container digital-view-arrow')
			.append(img1, img2)
			.mousedown(this._onArrowMouseDown.bind(this))
			.mouseup(this._onArrowMouseUp.bind(this))
			.mouseleave(this._onArrowMouseLeave.bind(this))
			.appendTo(this.main);
		arrow.img1 = img1;
		arrow.img2 = img2;
		return arrow;
	},

	createInput: function() {
		return $('<input/>')
			.addClass('digital-text')
			.mousedown(this._onInputMouseDown.bind(this))
			.focusout(this._onInputFocusOut.bind(this))
			.keydown(this._onKeyDown.bind(this))
			.appendTo(this.box);
	},

	createColon: function() {
		return $('<div/>')
			.addClass('digital-colon')
			.text(':')
			.appendTo(this.box);
	},

	//Props
	hours: function() {
		var text = this.inputs[0].val();
		return Number(text);
	},

	mins: function() {
		var text = this.inputs[1].val();
		return Number(text);
	},

	secs: function() {
		var text = this.inputs[2].val();
		return Number(text);
	},

	hms: function() {
		return {
			h: this.hours(),
			m: this.mins(),
			s: this.secs(),
		};
	},

	setTime: function(hours, mins, secs) {
		var cursor = this.cursorPos(); //keep cursor position
		this.inputs[0].val(hours);
		this.inputs[1].val(this.padNumber(mins));
		this.inputs[2].val(this.padNumber(secs));
		this.setCursorPos(cursor);
	},
	
	setTextColor: function(color) {
		var els = [
			this.colon1,
			this.colon2,
			this.inputs[0],
			this.inputs[1],
			this.inputs[2],
		];
		for(var i = 0; i < els.length; i++) {
			els[i].css({
				color: color,
			});
		}
	},

	arrowsShown: function() {
		return this._arrowsShown;
	},

	setArrowsShown: function(arrowsShown) {
		//Arrows
		this._arrowsShown = arrowsShown;
		for(var i = 0; i < this.arrows.length; i++) {
			if(arrowsShown) {
				this.arrows[i].show();
			} else {
				this.arrows[i].hide();
			}
		}

		//Others
		this.setTextColor(arrowsShown ? 'black' : this._regTextColor);
		this.box.css({
			backgroundColor: arrowsShown ? 'white' : 'transparent',
		});
	},

	setRegTextColor: function(color) {
		this._regTextColor = color;
		this.setTextColor(this._arrowsShown ? 'black' : this._regTextColor);
	},

	setLightBtnsShown: function(shown) {
		for(var i = 0; i < this.arrows.length; i++) {
			var arrow = this.arrows[i];
			if(shown) {
				arrow.img1.show();
				arrow.img2.hide();
			} else {
				arrow.img1.hide();
				arrow.img2.show();
			}
		}
	},

	fillSpaces: function() {
		var zero = this.inputs[0]
		var val = zero.val();
		if(val.length === 0) {
			zero.val('0');
		}
		this._fillSpaces(1);
		this._fillSpaces(2);
	},

	_fillSpaces: function(index) {
		var val = this.inputs[index].val();
		if(val.length === 0) {
			this.inputs[index].val('00');
		} else if(val.length === 1) {
			this.inputs[index].val('0' + val);
		}
	},

	boundTimes: function() {
		this._bound(0, 9, "0");
		this._bound(1, 59, "00");
		this._bound(2, 59, "00");
	},

	_bound: function(inputIndex, max, zeroText) {
		var input = this.inputs[inputIndex];
		var n = Number(input.val());
		if(typeof n != "number") {
			input.val(zeroText);
		} else if(n > max) {
			input.val(max);
		}
	},

	//Cursor
	cursorPos: function() {
		var inp = this.focussedInputIndex();
		if(inp == -1) {
			return null;
		}
		return {
			inputIndex: inp,
			cursorIndex: this.inputs[inp][0].selectionStart,
		}
	},

	setCursorPos: function(pos) {
		if(pos) {
			this.setCursor(pos.inputIndex, pos.cursorIndex);
		}
	},

	setCursor: function(inputIndex, cursorIndex) {
		var input = this.inputs[inputIndex][0];
		if(input.setSelectionRange) {
			input.focus();
			input.setSelectionRange(cursorIndex, cursorIndex);
		} else if(input.createTextRange) {
			var range = input.createTextRange();
			range.collapse(true);
			range.moveEnd('character', cursorIndex);
			range.moveStart('character', cursorIndex);
			range.select();
		}
	},

	focussedInputIndex: function() {
		for(var i = 0 ; i < 3; i++) {
			if(this.inputs[i].is(':focus')) {
				return i;
			}
		}
		return -1;
	},

	//Layout
	setFontSize: function(size) {
		var els = [
			this.colon1,
			this.colon2,
			this.inputs[0],
			this.inputs[1],
			this.inputs[2],
		];
		for(var i = 0; i < els.length; i++) {
			els[i].css({
				fontSize: size,
			});
		}
	},

	setCenter: function(x, y) {
		this.main.css({
			left: x - this.main.width() / 2,
			top: y - this.main.height() / 2,
		});
	},

	layoutInputs: function(x0, x1, x2, width) {
		/* Inputs are all percentage values */
		this.inputs[0].css({
			right: x0 + '%',
			width: width + '%',
		});
		this.inputs[1].css({
			right: x1 + '%',
			width: 2 * width + '%',
		});
		this.inputs[2].css({
			right: x2 + '%',
			width: 2 * width + '%',
		});
	},

	layoutArrows: function(x0, x1, x2, y1, y2, width) {
		/* Inputs are all percentage values */
		var w1 = width / 100 * this.main.width();
		var yCoords = [y1, y2];
		var xCoords = [x0, x1, x2];
		for(var i = 0; i < this.arrows.length; i++) {
			var x = xCoords[i % 3];
			var y = i < 3 ? yCoords[0] : yCoords[1];
			this.arrows[i].css({
				width: w1,
				height: w1,
				right: (x - width / 2) + '%',
				top: y + '%',
			});
		}
	},

	updateBorder: function() {
		var b = 0.025 * this.main.width();
		this.box.css({
			borderRadius: b,
		});
	},

	padNumber: function(x) {
		if(x < 10) {
			return '0' + String(x);
		} else {
			return String(x)
		}
	},

	//Events
	_onArrowMouseDown: function(event) {
		if(this.onArrowMouseDown) {
			var index = this.findArrowByEl(event.currentTarget);
			this.onArrowMouseDown(this, index);
		}
	},

	_onArrowMouseUp: function(event) {
		if(this.onArrowMouseUp) {
			var index = this.findArrowByEl(event.currentTarget);
			this.onArrowMouseUp(this, index);
		}
	},

	_onArrowMouseLeave: function(event) {
		if(this.onArrowMouseLeave) {
			var index = this.findArrowByEl(event.currentTarget);
			this.onArrowMouseLeave(this, index);
		}
	},

	_onInputMouseDown: function(event) {
		if(this.onInputMouseDown) {
			this.onInputMouseDown(this, event);
		}
	},

	_onKeyDown: function(event) {
		if(this.onKeyDown) {
			this.onKeyDown(this, event);
		}
	},

	_onInputFocusOut: function(event) {
		if(this.onInputFocusOut) {
			this.onInputFocusOut(this, event);
		}
	},

	//Other
	handleKeyDown: function(event) {
		/*
		Return true if time was changed
		*/
		var k = event.which;
		var cursorPos = this.cursorPos();
		var inputIndex = cursorPos.inputIndex;
		var cursorIndex = cursorPos.cursorIndex;
		var preHms = this.hms();

		//Defaults
		event.preventDefault();

		//Tab Press
		if(k === 9) {
			if(event.shiftKey) {
				this.selectInput(inputIndex - 1);
			} else {
				this.selectInput(inputIndex + 1);
			}
		}

		//Arrows
		if(k === 37 || k === 39) {
			this.moveCursorBy(k - 38);
		}

		//Numbers
		if(k >= 48 && k <= 57) {
			var jump = this.handleNumber(k, inputIndex, cursorIndex);
			if(jump) {
				this.selectInput(inputIndex + 1);
			}
		}

		//Delete
		if(k === 46 || k === 8) {
			if(!this.deleteSelectedRange()) {
				if(cursorIndex > 0) {
					this.deleteRange(inputIndex, cursorIndex -1, cursorIndex);					
				}
			}
		}

		this.boundTimes();
		var hms = this.hms();
		return preHms.h !== hms.h || preHms.m !== hms.m || preHms.s !== hms.s;
	},

	handleNumber: function(keyCode, inputIndex, cursorIndex) {
		this.deleteSelectedRange();
		var input = this.inputs[inputIndex];
		var val = input.val();
		var n = keyCode - 48;
		if(inputIndex === 0) {
			if(val.length === 0) {
				input.val(n);		
			}
			return val.length === 0;
		} else {
			if(val.length <= 1) {
				var input = this.inputs[inputIndex];
				input.val(cursorIndex === 0 ? n + input.val() : input.val() + n);
				this.setCursor(inputIndex, cursorIndex + 1);
				
			}
			return val.length === 1;
		}
	},

	moveCursorBy: function(n) {
		//Calc current total
		var cursorPos = this.cursorPos();
		var inputIndex = cursorPos.inputIndex;
		var cursorIndex = cursorPos.cursorIndex;
		var total = cursorIndex;
		if(inputIndex === 1) {
			total += 2;
		} else if(inputIndex === 2) {
			total += 5;
		}

		//Increment
		total += n;
		if(total < 0) {
			total = 0;
		} else if(total > 7) {
			total = 7;
		}

		//Move
		if(total < 2) {
			this.setCursor(0, total);
		} else if(total < 5) {
			this.setCursor(1, total - 2);
		} else {
			this.setCursor(2, total - 5);
		}
	},

	handleNumberKey: function(n, inputIndex, cursorIndex) {

	},

	selectInput: function(index) {
		if(index < 0 || index >= 3) {
			return;
		}
		this.inputs[index][0].select();
	},

	deleteSelectedRange: function() {
		var index = this.focussedInputIndex();
		var inp = this.inputs[index];
		var start = inp[0].selectionStart;
		var end = inp[0].selectionEnd;
		if(end - start > 0) {
			this.deleteRange(index, start, end);
			return true;
		}
		return false;
	},

	deleteRange: function(inputIndex, start, end) {
		var inp = this.inputs[inputIndex];
		var text = inp.val();
		inp.val(text.substring(0, start) + text.substring(end, text.length))
		this.setCursor(inputIndex, start);
	},

	findArrowByEl: function(el) {
		for(var i = 0; i < this.arrows.length; i++) {
			var a = this.arrows[i];
			if(a[0] === el) {
				return i;
			}
		}
		return -1;
	},


};