/*
Encapsulates a drop down menu with a title

Properties
	main: JQ -- main wrapper
	triangle: JQ -- triangle icon
	title: JQ
	box: JQ -- main option box which shows which option is selected.
	options: JQ[] -- divs representing each possible option. When a option is selected
		the corresponding div in this array is hidden and then the boxText is assigned to
		the new value
	_selectedIndex: Int -- index of selected option in options array
	onClick: function(DropView v, Event e) -- Convenience handler for when box is selected.
	onOptionClick: function(DropView v, Int index, Event e) -- Convenience handler for when option
		is selected.
	_isOptionsShown: Bool

 */

var DropView = function(title, options) {
	/* 
	Input
		title: String
		options: String[]
	*/
	//Properties
	this.triangle = createCenteredImg('img/down.png');
	this.triangle.img.width('80%')
		.height('80%');
	this.triangle.attr('draggable', false)
		.css({
			pointerEvents: 'none',
		});
	this.title = $('<div/>')
		.addClass('drop-title')
		.text(title);
	this.box = $('<div/>')
		.addClass('drop-option')
		.click(this._onClick.bind(this));
	this.main = $('<div/>')
		.addClass('drop-main')
		.append(this.title, this.box, this.triangle);
	this.options = []; //includes selected option
	this._selectedIndex = 0;
	this.onClick = null;
	this.onOptionClick = null;
	this._isOptionsShown = false;
	this.borderWidth = 2;

	//Initialize
	for(var i = 0; i < options.length; i++) {
		this.options[i] = $('<div/>')
			.addClass('drop-option')
			.text(options[i])
			.click(this._onOptionClick.bind(this))
			.appendTo(this.main);
	}
	this.setSelectedIndex(0);
	this.setOptionsShown(false);

};

DropView.prototype = {

	//Index
	selectedIndex: function() {
		return this._selectedIndex;
	},

	setSelectedIndex: function(selectedIndex) {
		if((typeof selectedIndex === 'number') && selectedIndex >= 0 && selectedIndex < this.options.length) {
			var old = this.options[this._selectedIndex]; 
			if(this._isOptionsShown) {
				old.show();
			}
			this.options[selectedIndex].hide();
			this._selectedIndex = selectedIndex;
			this.box.text(this.options[selectedIndex].text());
			this.layout();
		}
	},

	//Options
	isOptionsShown: function() {
		return this._isOptionsShown;
	},

	setOptionsShown: function(shown) {
		this._isOptionsShown = shown;
		for(var i = 0; i < this.options.length; i++) {
			if(shown && i !== this._selectedIndex) {
				this.options[i].show();
			} else {
				this.options[i].hide();
			}
		}
	},

	toggleOptions: function() {
		this.setOptionsShown(!this._isOptionsShown);
	},

	//Layout
	layoutLTWH: function(left, top, width, height) {
		this.main.css({
			left: left,
			top: top,
			width: width,
			height: height,
		});
		this.layout();
	},

	layout: function() {
		var h = this.main.height();
		var fontSize = 0.56 * h;
		var border = this.borderWidth + 'px solid white';
		var textPadX = 6;
		var padY = 0.18 * h;
		var padX = 0.3 * h;
		
		//Title
		this.title.css({
			top: -h * 0.9,
			fontSize: fontSize,
		});

		//Box
		var optionCss = {
			borderRadius: this.borderWidth,
			border: border,
			fontSize: fontSize,
			padding: padY + 'px ' + padX + 'px',
			height: h - 2 * padY,
			width: this.main.width() - 2 * padX,
		};
		this.box.css(optionCss)
			.css({
				left: -this.borderWidth,
				top: -this.borderWidth,
			});

		//Options
		var y = h;
		for(var i = 0; i < this.options.length; i++) {
			if(i != this._selectedIndex) {
				var o = this.options[i];
				o.css({
					left: -this.borderWidth,
					top: y,
				});
				o.css(optionCss);
				y += h + this.borderWidth;
			}
		}

		//Triangle
		var h1 = h * 0.5;
		var x1 = h * 0.25;
		this.triangle.css({
			width: h1,
			height: h1,
			top: x1,
			right: x1,
		});
	},

	//Events
	_onClick: function(event) {
		if(this.onClick) {
			this.onClick(this, event);
		}
	},

	_onOptionClick: function(event) {
		if(this.onOptionClick) {
			var selectedIndex = this.findOptionByEl(event.currentTarget);
			this.onOptionClick(this, selectedIndex, event);
		}
	},

	findOptionByEl: function(element) {
		for(var i = 0; i < this.options.length; i++) {
			if(this.options[i][0] === element) {
				return i;
			}
		}
		return -1;
	},

};
